Remove debug yield statements
[urcu.git] / urcu.c
diff --git a/urcu.c b/urcu.c
index d4a06844885cde3da2d03092e3e9c45a939e0438..018e09b8601b041621fc6f72afdaeb57839e895c 100644 (file)
--- a/urcu.c
+++ b/urcu.c
@@ -43,7 +43,9 @@ unsigned int __thread rand_yield;
 
 static struct reader_data *reader_data;
 static int num_readers, alloc_readers;
+#ifndef DEBUG_FULL_MB
 static int sig_done;
+#endif
 
 void internal_urcu_lock(void)
 {
@@ -74,34 +76,34 @@ static void switch_next_urcu_qparity(void)
        urcu_gp_ctr ^= RCU_GP_CTR_BIT;
 }
 
+#ifdef DEBUG_FULL_MB
+static void force_mb_all_threads(void)
+{
+       smp_mb();
+}
+#else
 static void force_mb_all_threads(void)
 {
        struct reader_data *index;
        /*
* Ask for each threads to execute a mb() so we can consider the
         * compiler barriers around rcu read lock as real memory barriers.
         */
        if (!reader_data)
                return;
-       debug_yield_write();
        sig_done = 0;
-       debug_yield_write();
-       mb();   /* write sig_done before sending the signals */
-       debug_yield_write();
-       for (index = reader_data; index < reader_data + num_readers; index++) {
+       smp_mb();       /* write sig_done before sending the signals */
+       for (index = reader_data; index < reader_data + num_readers; index++)
                pthread_kill(index->tid, SIGURCU);
-               debug_yield_write();
-       }
        /*
         * Wait for sighandler (and thus mb()) to execute on every thread.
         * BUSY-LOOP.
         */
        while (sig_done < num_readers)
                barrier();
-       debug_yield_write();
-       mb();   /* read sig_done before ending the barrier */
-       debug_yield_write();
+       smp_mb();       /* read sig_done before ending the barrier */
 }
+#endif
 
 void wait_for_quiescent_state(void)
 {
@@ -118,41 +120,62 @@ void wait_for_quiescent_state(void)
                while (rcu_old_gp_ongoing(index->urcu_active_readers))
                        barrier();
        }
-       /*
-        * Locally : read *index->urcu_active_readers before freeing old
-        * pointer.
-        * Remote (reader threads) : Order urcu_qparity update and other
-        * thread's quiescent state counter read.
-        */
-       force_mb_all_threads();
 }
 
-static void switch_qparity(void)
+void synchronize_rcu(void)
 {
-       /* All threads should read qparity before accessing data structure. */
-       /* Write ptr before changing the qparity */
+       /* All threads should read qparity before accessing data structure
+        * where new ptr points to. */
+       /* Write new ptr before changing the qparity */
        force_mb_all_threads();
-       debug_yield_write();
-       switch_next_urcu_qparity();
-       debug_yield_write();
+
+       internal_urcu_lock();
+
+       switch_next_urcu_qparity();     /* 0 -> 1 */
+
+       /*
+        * Must commit qparity update to memory before waiting for parity
+        * 0 quiescent state. Failure to do so could result in the writer
+        * waiting forever while new readers are always accessing data (no
+        * progress).
+        */
+       smp_mb();
 
        /*
         * Wait for previous parity to be empty of readers.
         */
-       wait_for_quiescent_state();
-}
+       wait_for_quiescent_state();     /* Wait readers in parity 0 */
+
+       /*
+        * Must finish waiting for quiescent state for parity 0 before
+        * committing qparity update to memory. Failure to do so could result in
+        * the writer waiting forever while new readers are always accessing
+        * data (no progress).
+        */
+       smp_mb();
+
+       switch_next_urcu_qparity();     /* 1 -> 0 */
+
+       /*
+        * Must commit qparity update to memory before waiting for parity
+        * 1 quiescent state. Failure to do so could result in the writer
+        * waiting forever while new readers are always accessing data (no
+        * progress).
+        */
+       smp_mb();
+
+       /*
+        * Wait for previous parity to be empty of readers.
+        */
+       wait_for_quiescent_state();     /* Wait readers in parity 1 */
 
-void synchronize_rcu(void)
-{
-       debug_yield_write();
-       internal_urcu_lock();
-       debug_yield_write();
-       switch_qparity();
-       debug_yield_write();
-       switch_qparity();
-       debug_yield_write();
        internal_urcu_unlock();
-       debug_yield_write();
+
+       /* All threads should finish using the data referred to by old ptr
+        * before decrementing their urcu_active_readers count */
+       /* Finish waiting for reader threads before letting the old ptr being
+        * freed. */
+       force_mb_all_threads();
 }
 
 void urcu_add_reader(pthread_t id)
@@ -217,9 +240,10 @@ void urcu_unregister_thread(void)
        internal_urcu_unlock();
 }
 
+#ifndef DEBUG_FULL_MB
 void sigurcu_handler(int signo, siginfo_t *siginfo, void *context)
 {
-       mb();
+       smp_mb();
        atomic_inc(&sig_done);
 }
 
@@ -249,3 +273,4 @@ void __attribute__((destructor)) urcu_exit(void)
        assert(act.sa_sigaction == sigurcu_handler);
        free(reader_data);
 }
+#endif
This page took 0.024648 seconds and 4 git commands to generate.
@@ -74,34 +76,34 @@ static void switch_next_urcu_qparity(void)
        struct reader_data *index;
        /*
-        * Ask for each threads to execute a mb() so we can consider the
+        * Ask for each threads to execute a smp_mb() so we can consider the
         * compiler barriers around rcu read lock as real memory barriers.
         */